/*

	Byte-order invariant definitions

*/

/*  The following macros allow packing fields of various types
    into a byte-oriented packet with a running pointer named
    pktP.  The reason we store then memcpy rather than storing
    through a cast of pktP to the desired type is that the
    latter approach can result in alignment faults when run
    on RISC machines which typically require values to be
    aligned at an even multiple of their length.  */

#define pack16(x) { \
			uint16 s = htons((uint16) (x)); \
			memcpy(pktP, &s, sizeof(uint16)); \
			pktP += sizeof(uint16); \
		     }

#define pack32(x) { \
			uint32 i = htonl((uint32) (x)); \
			memcpy(pktP, &i, sizeof(uint32)); \
			pktP += sizeof(uint32); \
		    }

#define pack8(x) { \
			char c = (x); \
			memcpy(pktP, &c, sizeof(char)); \
			pktP += sizeof(char); \
		    }

#define pack8s(x, n) { \
			    memcpy(pktP, x, n); \
			    pktP += n; \
			}

/*  These macros unpack fields of various lengths from a
    byte-packet packet in network byte order pointed to by
    pktP.  Note that the argument of these macros must be
    an lvalue.	*/

#define unpack16(x) { \
			    uint16 s; \
			    memcpy(&s, pktP, sizeof(uint16)); \
			    pktP += sizeof(uint16); \
			    x = ntohs(s); \
		       }

#define unpack32(x) { \
			   uint32 i; \
			   memcpy(&i, pktP, sizeof(uint32)); \
			   pktP += sizeof(uint32); \
			   x = ntohl(i); \
		      }

#define unpack8(x) { \
			*((char *) &x) = (char) *pktP++; \
		      }

#define unpack8s(x, n) { \
			    memcpy(x, pktP, n); \
			    pktP += n; \
			  }
